package meerkat;

import util.AccurateTimestamp;
import util.Tags;

import java.sql.Timestamp;
import java.text.ParseException;

/**
 * Created by Vladimir Eliezer Tokarev on 1/15.2016
 * this class wraps BasicMessage and gives the ability to find wanted tags
 */
public class VoterRegistryMessage {

    public ProtobufsMessages.BasicMessage base;

    public VoterRegistryMessage(ProtobufsMessages.BasicMessage message){
        base = ProtobufsMessages.BasicMessage.newBuilder().addAllTag(message.getTagList()).build();
    }

    /**
     * Gets the wanted tag from given basic message
     * @param tagName
     * @return string
     */
    public String GetWantedTagFromBasicMessage(Tags tagName){
        for (ProtobufsMessages.Tag tag :  base.getTagList()) {
            if ( tag.getContent().contains(tagName.toString())) {
                return tag.getContent();
            }
        }
        return null;
    }

    /**
     * Gets the timestamp of the tag adding
     * @return Timestamp
     * @throws ParseException
     */
    public  Timestamp GetBasicMessageActionTimestamp() throws ParseException {
        for (ProtobufsMessages.Tag tag : base.getTagList()) {
            if ( tag.getContent().contains(Tags.ACTION_TIMESTAMP_TAG.toString())) {
                String[] tagParts = tag.getContent().split(" ");
                String timestamp = tagParts[tagParts.length - 1];
                return AccurateTimestamp.GetTimestampFromString(timestamp);
            }
        }
        return null;

    }

    /**
     * Checks if the given message have the ADD_TO_GROUP_TAG
     * @return
     */
    public  boolean  IsGroupAdding() {
        for (ProtobufsMessages.Tag tag : base.getTagList()) {
            if ( tag.getContent().contains(Tags.ADD_TO_GROUP_TAG.toString())) {
                return true;
            }
        }
        return false;
    }
}
